use strict;
use warnings;

package Data::TestDspaceAPI;

use base 'Exporter';

use MongoDB::OID;
use Readonly;
use Perl6::Slurp;


our @EXPORT_OK = qw(
    $PUBLICATIONS_SINCE_1 $PUBLICATIONS_SINCE_TERMINATOR

    $SETS_1 $SETS_2 $SETS_3 $SETS_4

    $SET_SPEC_TO_NAME_MAPPING

    $PUBLICATION_1 $TITLE_TWEET_ITEM_1
    $PUBLICATION_2 $TITLE_TWEET_ITEM_2
    $PUBLICATION_3 $TITLE_TWEET_ITEM_3
    $PUBLICATION_4 $TITLE_TWEET_ITEM_4
    $PUBLICATION_5 $TITLE_TWEET_ITEM_5
    $PUBLICATION_6 $TITLE_TWEET_ITEM_6
    $PUBLICATION_7 $TITLE_TWEET_ITEM_7
);


Readonly our $PUBLICATIONS_SINCE_1 => {
    status => 200,
    response => scalar slurp 't/Data/DspaceAPI/publications_1.xml',
};

Readonly our $PUBLICATIONS_SINCE_TERMINATOR => {
    status => 200,
    response => scalar slurp 't/Data/DspaceAPI/publications_2.xml',
};

Readonly our $SETS_1 => {
    status => 200,
    response => scalar slurp 't/Data/DspaceAPI/sets_1.xml',
};

Readonly our $SETS_2 => {
    status => 200,
    response => scalar slurp 't/Data/DspaceAPI/sets_2.xml',
};

Readonly our $SETS_3 => {
    status => 200,
    response => scalar slurp 't/Data/DspaceAPI/sets_3.xml',
};

Readonly our $SETS_4 => {
    status => 200,
    response => scalar slurp 't/Data/DspaceAPI/sets_4.xml',
};


Readonly our $PUBLICATION_1 => {
    dspaceId => 'oai:dspace.lboro.ac.uk:2134/1238',
    sets => [
        {id => 'com_2134_92', name => 'Physics'},
        {id => 'com_2134_80', name => 'Science'},
        {id => 'col_2134_1134', name => 'Pre-Prints (Physics)'},
    ],
    title => "Absolute negative conductivity and spontaneous current generation in semiconductor superlattices with hot electrons",
    authors => [
        "Cannon, E.H.",
        "Kusmartsev, F.V.",
        "Alekseev, K.N.",
        "Campbell, D.K.",
    ],
    tags => [
        "mesoscopic systems",
        "quantum hall effect",
    ],
    descriptions => [
        "This is a pre-print. It is also available at: http://arxiv.org/abs/cond-mat/9910042.",
        "We study electron transport through a semiconductor superlattice subject to an electric field parallel to and a magnetic field perpendicular to the growth axis. Using a single miniband, semiclassical balance equation model with both elastic and inelastic scattering, we find that (1) the current-voltage characteristic becomes multistable in a large magnetic field; and (2) \"hot\" electrons display novel features in their current-voltage characteristics, including absolute negative conductivity (ANC) and, for sufficiently strong magnetic fields, a spontaneous dc current at zero bias. We discuss possible experimental situations providing the necessary hot electrons to observe the predicted ANC and spontaneous dc current generation.",
    ],
    type => ['Preprint'],
    url => 'https://dspace.lboro.ac.uk/2134/1238',
    language => 'en',
};

Readonly our $TITLE_TWEET_ITEM_1 => "#publication Absolute negative conductivity and spontaneous current generation in semiconductor superlattices with hot electrons https://dspace.lboro.ac.uk/2134/1238";

Readonly our $PUBLICATION_2 => {
    dspaceId => 'oai:dspace.lboro.ac.uk:2134/1310',
    sets => [
        {id => 'com_2134_92', name => 'Physics'},
        {id => 'com_2134_80', name => 'Science'},
        {id => 'col_2134_93', name => 'Published Articles (Physics)'},
    ],
    title => "Coherence of the lattice polarization in large-polaron motion",
    authors => [
        "Myasnikov, E.N.",
        "Myasnikova, A.E.",
        "Kusmartsev, F.V.",
    ],
    descriptions => [
        "This article was published in the journal, Physical Review B [?? American Physical Society].  It is also available at: http://link.aps.org/doi/10.1103/PhysRevB.72.224303.",
        "Main problems of the large polaron theory are considered. We demonstrate that the problem of searching the ground stationary state of a system of coupled fields with translation-invariant Hamiltonian can have a solution of the form f(r???vt), v???0, i.e., the solution with the spontaneously broken translational symmetry. Such a state can be a ground state of a large polaron in case of strong electron-phonon coupling when the spontaneous break of the translational symmetry results from the phonon vacuum deformation by the electric field of the charge carrier. The correctness of the classical representation of the polarization field in the theory of a strongly coupled large polaron is proved on the base of the theory of the quantum-coherent states of the phonon field. The use of this representation has enabled us to show that extremely high losses of the electron energy in dielectric parts of cold cathodes occurring when the carrier velocity is lower than the threshold for the single-phonon radiation are due to coherent phonon radiation by polarons like Cherenkov effect. It is this radiation that results in the predicted Thornber and Feynman dependence of the carrier steady-state velocity on the applied electric field strength. The coherent phonon radiation generated by polaron current can be detected in experiments on the neutrons scattering. The primary directions of the neutrons scattering depend on the polarons steady-state velocity and, hence, on the applied field strength. The coherent phonon radiation stemming from supersonic thermal motion of polarons causes a giant increase of the resistance in a corresponding temperature interval.",
    ],
    type => ['Article'],
    url => 'https://dspace.lboro.ac.uk/2134/1310',
    language => 'en',
};

Readonly our $TITLE_TWEET_ITEM_2 => "#publication Coherence of the lattice polarization in large-polaron motion https://dspace.lboro.ac.uk/2134/1310";

Readonly our $PUBLICATION_3 => {
    dspaceId => 'oai:dspace.lboro.ac.uk:2134/2122',
    sets => [
        {id => 'com_2134_92', name => 'Physics'},
        {id => 'com_2134_80', name => 'Science'},
        {id => 'col_2134_1134', name => 'Pre-Prints (Physics)'},
    ],
    title => "Buckyball quantum computer: realization of a quantum gate",
    authors => [
        'Garelli, M.S.',
        'Kusmartsev, F.V.',
    ],
    descriptions => [
        "This is a pre-print.  It is also available at: http://arxiv.org/abs/quant-ph/0501076.",
        "We have studied a system composed by two endohedral fullerene molecules. We have found that\r\nthis system can be used as good candidate for the realization of Quantum Gates. Each of these molecules\r\nencapsules an atom carrying a spin, therefore they interact through the spin dipole interaction. We show\r\nthat a phase gate can be realized if we apply static and time dependent magnetic fields on each encased\r\nspin. We have evaluated the operational time of a pi-phase gate, which is of the order of ns. We made a\r\ncomparison between the theoretical estimation of the gate time and the experimental decoherence time for\r\neach spin. The comparison shows that the spin relaxation time is much larger than the pi-gate operational\r\ntime. Therefore, this indicates that, during the decoherence time, it is possible to perform some thousands\r\nof quantum computational operations. Moreover, through the study of concurrence, we get very good\r\nresults for the entanglement degree of the two-qubit system. This finding opens a new avenue for the\r\nrealization of Quantum Computers.",
    ],
    type => ['Preprint'],
    url => 'https://dspace.lboro.ac.uk/2134/2122',
    language => 'en',
};

Readonly our $TITLE_TWEET_ITEM_3 => "#publication Buckyball quantum computer: realization of a quantum gate https://dspace.lboro.ac.uk/2134/2122";

Readonly our $PUBLICATION_4 => {
    dspaceId => 'oai:dspace.lboro.ac.uk:2134/2309',
    sets => [
        {id => 'com_2134_66', name => 'Aeronautical and Automotive Engineering'},
        {id => 'com_2134_65', name => 'Aeronautical, Automotive, Chemical and Materials Engineering'},
        {id => 'col_2134_73', name => 'Published Articles (Aeronautical and Automotive Engineering)'},
    ],
    title => "Genetic algorithm optimisation of a firewater deluge system",
    authors => [
        "Andrews, J.D.",
        "Bartlett, L.M.",
    ],
    tags => [
        'Optimisation',
        'Genetic algorithms',
        'Safety systems',
        'Design',
    ],
    descriptions => [
        'This article was published in the journal, Quality and Reliability Engineering International [?? John Wiley & Sons] and is also available at http://www3.interscience.wiley.com/cgi-bin/jhome/3680?CRETRY=1&SRETRY=0',
        'Safety systems are designed to operate when certain conditions occur and act to prevent their development into a hazardous situation. Failure of a safety system for a potentially hazardous industrial system or process may have catastrophic consequences, possibly injuring members of the work force or public and occasionally resulting in loss of life. The purpose of this paper is to describe a design optimisation scheme using genetic algorithms applied to a firewater deluge system, which uses available resources to the best possible advantage to obtain an optimal safety system design.',
    ],
    type => ['Article'],
    url => 'https://dspace.lboro.ac.uk/2134/2309',
    language => 'en',
};

Readonly our $TITLE_TWEET_ITEM_4 => "#publication Genetic algorithm optimisation of a firewater deluge system";

Readonly our $PUBLICATION_5 => {
    dspaceId => 'oai:dspace.lboro.ac.uk:2134/2399',
    sets => [
        {id => 'com_2134_66', name => 'Aeronautical and Automotive Engineering'},
        {id => 'com_2134_65', name => 'Aeronautical, Automotive, Chemical and Materials Engineering'},
        {id => 'col_2134_73', name => 'Published Articles (Aeronautical and Automotive Engineering)'},
    ],
    title => "Using statistically designed experiments for safety system optimization",
    authors => [
        'Bartlett, L.M.',
        'Andrews, J.D.',
    ],
    tags => [
        'Statistically designed experiments',
        'Fault tree analysis',
        'Binary decision systems',
        'Optimisation',
        'Safety systems',
    ],
    descriptions => [
        "This article was published in the journal, Proceedings of the Institution of Mechanical Engineers, Part E: Journal of Process Mechanical Engineering [?? Professional Engineering Publishing] and is also available at: http://pep.metapress.com/content/119780",
        "This paper describes the method of statistically designed experiments (SDE's), used as a\r\nstructured method to investigate the best setting for a number of decision variables in a\r\nsystem design problem. Traditionally, in the design of safety critical systems, a trial and\r\nerror type approach is undertaken to achieve a final system that meets the design\r\nobjectives. This approach can be time consuming and often only an adequate design is\r\nfound rather than the optimal design for the available resources. Optimal use of\r\nresources should be imperative when possible lives are at risk. To demonstrate the\r\npracticality of this new structured approach for optimising a safety system design, a high\r\nintegrity safety system has been used. Each design is analysed using the Binary Decision\r\nDiagram analysis technique to establish the system unavailability, which is penalised if\r\nthe system constraints are exceeded. System constraints indicate the limitations on the resources which can be utilised. The SDE approach highlights good and bad settings for\r\npossible design variables. This knowledge can then be used by more sophisticated search\r\ntechniques. The latter part of this paper analyses the results from the best design\r\ngenerated using the SDE, for further optimisation using localised optimisation\r\napproaches.",
    ],
    type => ['Article'],
    url => 'https://dspace.lboro.ac.uk/2134/2399',
    doi => '10.1243/095440804322860645',
    language => 'en',
};


Readonly our $TITLE_TWEET_ITEM_5 => "#publication Using statistically designed experiments for safety system optimization https://dspace.lboro.ac.uk/2134/2399";

Readonly our $PUBLICATION_6 => {
    dspaceId => 'oai:dspace.lboro.ac.uk:2134/3536',
    sets => [
        {id => 'com_2134_81', name => 'Chemistry'},
        {id => 'com_2134_80', name => 'Science'},
        {id => 'col_2134_3149', name => 'Closed Access (Library)'},
    ],
    title => "Methyl 4-ethoxy-2-methyl-2H-1,2-benzothiazine-3-carboxylate 1,1-dioxide",
    authors => [
        'Zia-ur-Rehman, Muhammad',
        'Choudary, Jamil Anwar',
        'Elsegood, Mark R.J.',
        'Akbar, N.',
        'Siddiqui, Hamid Latif',
    ],
    descriptions => [
        'This article was published in the journal, Acta Crystallographica Section E: Structure Reports Online [?? International Union of Crystallography]. It is also available at: http://journals.iucr.org/e/',
    ],
    type => ['Article'],
    url => 'https://dspace.lboro.ac.uk/2134/3536',
    language => 'en',
};

Readonly our $TITLE_TWEET_ITEM_6 => "#publication Methyl 4-ethoxy-2-methyl-2H-1,2-benzothiazine-3-carboxylate 1,1-dioxide https://dspace.lboro.ac.uk/2134/3536";

Readonly our $PUBLICATION_7 => {
    dspaceId => 'oai:dspace.lboro.ac.uk:2134/18261',
    sets => [
        {id => 'com_2134_89', name => 'Mathematical Sciences'},
        {id => 'com_2134_80', name => 'Science'},
        {id => 'col_2134_3149', name => 'Closed Access (Library)'},
    ],
    title => "The SEFI Maths Working Group: current offerings and future tasks",
    authors => [
        'Alpers, Burkhard A.',
        'Demlova, Marie',
        'Gustafsson, Tommy',
        'Lawson, Duncan',
        'Olsson-Lehtonen, Brita',
        'Robinson, Carol',
        'Robinson, Paul',
        'Velichova, Daniela',
    ],
    descriptions => [
        'This is a conference paper.',
        "In this discussion paper we firstly summarise the current offering of the SEFI Mathematics\r\nWorking Group with regard to orientation for those who are interested in the mathematical\r\neducation of engineers. Based on this summary we identify directions for further work. Finally,\r\nwe present some ideas of how progress might be made in these directions.",
    ],
    type => ['Conference Contribution'],
    url => 'https://dspace.lboro.ac.uk/2134/18261',
    language => 'en',
};

Readonly our $TITLE_TWEET_ITEM_7 => "#publication The SEFI Maths Working Group: current offerings and future tasks https://dspace.lboro.ac.uk/2134/18261";

Readonly our $SET_SPEC_TO_NAME_MAPPING => {
    com_2134_65 => 'Aeronautical, Automotive, Chemical and Materials Engineering',
    com_2134_66 => 'Aeronautical and Automotive Engineering',
    com_2134_67 => 'Chemical Engineering',
    com_2134_68 => 'Civil and Building Engineering',
    com_2134_69 => 'Electronic, Electrical and Systems Engineering',
    com_2134_71 => 'Mechanical and Manufacturing Engineering',
    com_2134_80 => 'Science',
    com_2134_81 => 'Chemistry',
    com_2134_83 => 'Computer Science',
    com_2134_87 => 'Information Science',
    com_2134_89 => 'Mathematical Sciences',
    com_2134_92 => 'Physics',
    com_2134_94 => 'Social, Political and Geographical Sciences',
    com_2134_95 => 'Business School',
    com_2134_97 => 'Design',
    com_2134_99 => 'Economics',
    com_2134_102 => 'English and Drama',
    com_2134_103 => 'Politics, History and International Relations',
    com_2134_104 => 'Geography',
    com_2134_105 => 'Sport, Exercise and Health Sciences',
    com_2134_106 => 'Arts',
    com_2134_108 => 'Social Sciences',
    com_2134_111 => 'University Research Centres',
    com_2134_127 => 'Centre for Child and Family Research',
    com_2134_129 => 'Centre for Hazard and Risk Management (CHaRM)',
    com_2134_130 => 'Centre for Innovative and Collaborative Engineering (CICE) ',
    com_2134_132 => 'Centre for Non-Linear Mathematics & Applications',
    com_2134_134 => 'Centre for Renewable Energy Systems Technology (CREST)',
    com_2134_135 => 'Centre for Research in Social Policy (CRSP)',
    com_2134_144 => 'Innovative Manufacturing and Construction Research Centre',
    com_2134_147 => 'Loughborough Sleep Research Centre',
    com_2134_148 => 'Professional and Management Development Centre',
    com_2134_150 => 'Midlands Centre for Criminology and Criminal Justice ',
    com_2134_154 => 'Systems Engineering Innovation Centre (SEIC)',
    com_2134_158 => 'Water, Engineering and Development Centre (WEDC)',
    com_2134_165 => 'University Academic and Administrative Support',
    com_2134_166 => 'University Library',
    com_2134_510 => 'Materials',
    com_2134_1758 => 'Professional Development',
    com_2134_2156 => 'engCETL',
    com_2134_3066 => 'University Research Groups',
    com_2134_3073 => 'Design Education Research Group',
    com_2134_3075 => 'Control Systems Group',
    com_2134_3088 => 'IT Services',
    com_2134_8512 => 'Higher Education Academy Engineering Subject Centre',
    com_2134_8534 => 'Mathematics Education Centre',
    com_2134_8699 => 'The Arts, English and Drama',
    com_2134_8700 => 'Business and Economics',
    com_2134_8803 => 'Research Office',
    com_2134_9373 => 'Centre for Engineering and Design Education (CEDE)',
    com_2134_9781 => 'Design Practice Research Group',
    com_2134_13999 => 'Centre for Information Management (CIM)',
    com_2134_19016 => 'Loughborough University London',
    com_2134_19492 => 'Sports Technology Research Group',
    col_2134_73 => 'Published Articles (Aeronautical and Automotive Engineering)',
    col_2134_74 => 'Published Articles (Chemical Engineering)',
    col_2134_75 => 'Published Articles (Civil and Building Engineering)',
    col_2134_76 => 'Published Articles (Electronic, Electrical and Systems Engineering)',
    col_2134_78 => 'Published Articles (Mechanical and Manufacturing Engineering)',
    col_2134_82 => 'Published Articles (Chemistry)',
    col_2134_84 => 'Published Articles (Computer Science)',
    col_2134_88 => 'Published Articles (Information Science)',
    col_2134_90 => 'Published Articles (Maths)',
    col_2134_91 => 'Pre-prints (Maths)',
    col_2134_93 => 'Published Articles (Physics)',
    col_2134_96 => 'Published Articles (Business School)',
    col_2134_98 => 'Published Articles (Design School)',
    col_2134_100 => 'Published Articles (Economics)',
    col_2134_101 => 'Working Papers (Economics)',
    col_2134_107 => 'Moving Images (Arts)',
    col_2134_159 => 'Published Articles (English and Drama)',
    col_2134_161 => 'Published Articles (Geography)',
    col_2134_162 => 'Published Articles (PHIR)',
    col_2134_163 => 'Published Articles (Social Sciences)',
    col_2134_164 => 'Published Articles (Sport, Exercise and Health Sciences)',
    col_2134_167 => 'Published Articles (Library)',
    col_2134_168 => 'Annual Reports (Library)',
    col_2134_255 => 'Published Articles (Sleep Research)',
    col_2134_340 => 'Book Chapters (English and Drama)',
    col_2134_342 => 'Research Reports (Information Science)',
    col_2134_407 => 'Conference Papers (Information Science)',
    col_2134_470 => 'Pre-Prints (Information Science)',
    col_2134_481 => 'Conference Papers and Presentations (Computer Science)',
    col_2134_570 => 'MSc and MA Dissertations (Information Science)',
    col_2134_606 => 'Official Reports (Social Sciences)',
    col_2134_617 => 'Book Chapters (Social Sciences)',
    col_2134_658 => 'Book Chapters (Mechanical and Manufacturing Engineering)',
    col_2134_696 => 'Conference Papers and Contributions (Mechanical and Manufacturing Engineering)',
    col_2134_712 => 'Working Papers (Mechanical and Manufacturing Engineering)',
    col_2134_778 => 'Published Articles (Criminology)',
    col_2134_786 => 'Conference Papers and Presentations (Social Sciences)',
    col_2134_791 => 'Published Theses (CICE)',
    col_2134_806 => 'Published Articles (Materials)',
    col_2134_851 => 'Official Reports (Criminology)',
    col_2134_885 => 'Conference Papers (Economics)',
    col_2134_927 => 'Book Chapters (Criminology)',
    col_2134_951 => 'Books (Design School)',
    col_2134_952 => 'Conference Papers and Contributions (Design School)',
    col_2134_954 => 'IDATER Archive',
    col_2134_1121 => 'Pre-prints (Library)',
    col_2134_1134 => 'Pre-Prints (Physics)',
    col_2134_1147 => 'PhD Theses (Business School)',
    col_2134_1213 => 'Conference Papers (Business School)',
    col_2134_1289 => 'Conference Papers (English and Drama)',
    col_2134_1641 => 'Conference Papers (PHIR)',
    col_2134_1680 => 'Closed Access (Physics)',
    col_2134_1686 => 'Orange Series',
    col_2134_1759 => 'CAA Conference',
    col_2134_1896 => 'Degree Show (Arts)',
    col_2134_2012 => 'Occasional Papers Series (Business School)',
    col_2134_2037 => 'Closed Access (Mechanical and Manufacturing Engineering)',
    col_2134_2040 => 'Closed Access (Chemistry)',
    col_2134_2042 => 'Closed Access (Economics)',
    col_2134_2090 => 'PhD Theses (Geography)',
    col_2134_2091 => 'Closed Access (English and Drama)',
    col_2134_2096 => 'Closed Access (PHIR)',
    col_2134_2099 => 'Official Reports (Business School)',
    col_2134_2102 => 'Conference Papers (WEDC)',
    col_2134_2134 => 'Closed Access (Social Sciences)',
    col_2134_2157 => 'Published Articles (engCETL)',
    col_2134_2175 => 'PhD Theses (Design School)',
    col_2134_2208 => 'PhD Theses (Materials)',
    col_2134_2211 => 'Closed Access (Sport, Exercise and Health Sciences)',
    col_2134_2239 => 'Conference Papers (Civil and Building Engineering)',
    col_2134_2241 => 'Closed Access (Civil and Building Engineering)',
    col_2134_2256 => 'Conference Papers (CRSP)',
    col_2134_2257 => 'CRSP Research Reports',
    col_2134_2258 => 'Published Articles (CRSP)',
    col_2134_2259 => 'Closed Access (CRSP)',
    col_2134_2260 => 'Official Reports (CRSP)',
    col_2134_2270 => 'PhD Theses (Computer Science)',
    col_2134_2279 => 'Conference Papers and Presentations (Chemistry)',
    col_2134_2295 => 'PhD Theses (Civil and Building Engineering)',
    col_2134_2340 => 'Conference Papers (Library)',
    col_2134_2342 => 'Books (PHIR)',
    col_2134_2371 => 'Book Chapters (PHIR)',
    col_2134_2376 => 'Pre-Prints (Computer Science)',
    col_2134_2396 => 'Closed Access (Computer Science)',
    col_2134_2397 => 'Closed Access (Aeronautical and Automotive Engineering)',
    col_2134_2408 => 'Pre-Prints (Geography)',
    col_2134_2413 => 'NADE Archive',
    col_2134_2442 => 'Official Reports (Sport, Exercise and Health Sciences)',
    col_2134_2447 => 'Conference Papers (Physics)',
    col_2134_2484 => 'Closed Access (Maths)',
    col_2134_2509 => 'Official Reports (Library)',
    col_2134_2512 => 'Pre-Prints (Social Sciences)',
    col_2134_2577 => 'Closed Access (Materials)',
    col_2134_2725 => 'PhD Theses (Arts)',
    col_2134_2727 => 'Book Chapters (Design School)',
    col_2134_2767 => 'D&T Association Conference Series',
    col_2134_2807 => 'Pre-Prints (Chemistry)',
    col_2134_2809 => 'PhD Theses (Social Sciences)',
    col_2134_2819 => 'PhD Theses (Sport, Exercise and Health Sciences)',
    col_2134_2889 => 'Book Chapters (Arts)',
    col_2134_2897 => 'Conference Papers and Contributions (Electronic, Electrical and Systems Engineering)',
    col_2134_2905 => 'Conference Papers and Contributions (Arts)',
    col_2134_2911 => 'Official Reports (Centre for Child and Family Research)',
    col_2134_2921 => 'Evidence Papers (Centre for Child and Family Research)',
    col_2134_2944 => 'Published Articles (Arts)',
    col_2134_2974 => 'PhD Theses (Information Science)',
    col_2134_3008 => 'Closed Access (Information Science)',
    col_2134_3012 => 'Technical Reports (Electronic, Electrical and Systems Engineering)',
    col_2134_3021 => 'Research Reports (engCETL)',
    col_2134_3028 => 'Pre-Prints (Materials)',
    col_2134_3054 => 'PhD Theses (PHIR)',
    col_2134_3069 => 'Closed Access (Electronic, Electrical and Systems Engineering)',
    col_2134_3077 => 'Conference Papers (Control Systems Group)',
    col_2134_3078 => 'Technical Papers (Control Systems Group)',
    col_2134_3090 => 'Conference Papers (IT Services)',
    col_2134_3097 => 'Closed Access (Business School)',
    col_2134_3107 => 'Book Chapters (Chemistry)',
    col_2134_3111 => 'Data Sets and Software (CREST)',
    col_2134_3117 => 'Conference Papers and Contributions (CREST)',
    col_2134_3149 => 'Closed Access (Library)',
    col_2134_3231 => 'Research Reports (Business School)',
    col_2134_3260 => 'Book Chapters (Library)',
    col_2134_3302 => 'Closed Access (Geography)',
    col_2134_3306 => 'Pre-Prints (Electronic, Electrical and Systems Engineering)',
    col_2134_3308 => 'Pre-Prints (Control Systems Group)',
    col_2134_3309 => 'Official Reports (Civil and Building Engineering)',
    col_2134_3358 => 'Closed Access (Design School)',
    col_2134_3375 => 'Book Chapters (Civil and Building Engineering)',
    col_2134_3511 => 'Conference Papers and Presentations (Centre for Child and Family Research)',
    col_2134_3551 => 'Closed Access (Chemical Engineering)',
    col_2134_3557 => 'PhD Theses (Electronic, Electrical and Systems Engineering)',
    col_2134_3605 => 'Published Articles (WEDC)',
    col_2134_3610 => 'Conference Papers and Contributions (Aeronautical and Automotive Engineering)',
    col_2134_3771 => 'Conference Papers (Geography)',
    col_2134_3867 => 'Closed Access (WEDC)',
    col_2134_3919 => 'Design Work (Arts)',
    col_2134_3933 => 'Book Chapters (Chemical Engineering)',
    col_2134_4033 => 'PhD Theses (Chemical Engineering)',
    col_2134_4066 => 'Data Sets and Software (Electronic, Electrical and Systems Engineering)',
    col_2134_4077 => 'PhD Theses (Mechanical and Manufacturing Engineering)',
    col_2134_4160 => 'Closed Access (Criminology)',
    col_2134_4234 => 'Other (Civil and Building Engineering)',
    col_2134_4318 => 'Pre-Prints (Business School)',
    col_2134_4335 => 'Book Chapters (Electronic, Electrical and Systems Engineering)',
    col_2134_4396 => 'Book Chapters (Geography)',
    col_2134_4433 => 'PhD Theses (English and Drama)',
    col_2134_4472 => 'Conference Papers and Presentations (Sport, Exercise and Health Sciences)',
    col_2134_4475 => 'Book Chapters (Computer Science)',
    col_2134_4598 => 'Conference Papers and Presentations (Materials)',
    col_2134_4624 => 'Pre-Prints (Mechanical and Manufacturing Engineering)',
    col_2134_4640 => 'PhD Theses (Chemistry)',
    col_2134_4642 => 'PhD Theses (Maths)',
    col_2134_4740 => 'Conference Papers and Presentations (Chemical Engineering)',
    col_2134_4758 => 'Published Articles (CREST)',
    col_2134_4920 => 'Imported Works',
    col_2134_4927 => 'Reports (CREST)',
    col_2134_4967 => 'Published Letters (Chemical Engineering)',
    col_2134_5021 => 'Published Articles (CICE)',
    col_2134_5345 => 'Conference Papers and Presentations (engCETL)',
    col_2134_5355 => 'Interrogations Conference Papers (Arts)',
    col_2134_5411 => 'Published Articles (Control Systems Group)',
    col_2134_5413 => 'Book Chapters (Control Systems Group)',
    col_2134_5468 => 'Pre-Prints (Chemical Engineering)',
    col_2134_5473 => 'Book Chapters (Sport, Exercise and Health Sciences) ',
    col_2134_5529 => 'PhD Theses (Control Systems Group)',
    col_2134_5565 => 'PhD Theses (engCETL) ',
    col_2134_5766 => 'Pre-Prints (Sport, Exercise and Health Sciences)',
    col_2134_5807 => 'PhD Theses (Aeronautical and Automotive Engineering) ',
    col_2134_5977 => 'Published Letters (Chemistry) ',
    col_2134_6079 => 'Closed Access (Arts)',
    col_2134_6220 => 'Books (English and Drama)',
    col_2134_6412 => 'Socio-economic status and child/adolescent health in Johannesburg-Soweto Study',
    col_2134_6586 => 'Closed Access (CICE)',
    col_2134_6619 => 'Working Papers (Geography)',
    col_2134_6623 => 'PhD Theses (Physics)',
    col_2134_6624 => 'PhD Theses (Economics)',
    col_2134_6625 => 'MPhil Theses (Sport, Exercise and Health Sciences)',
    col_2134_6706 => 'Official Reports (Design School)',
    col_2134_6719 => 'MSc Dissertations (Design School)',
    col_2134_6720 => 'EThOS',
    col_2134_7485 => 'Research Reports (Social Sciences)',
    col_2134_7658 => 'PhD Theses (CHaRM)',
    col_2134_7733 => 'Audio Media (Arts)',
    col_2134_7844 => 'Book Chapters (Business School)',
    col_2134_7846 => 'MPhil Theses (Design School)',
    col_2134_7874 => 'MPhil Theses (Business School)',
    col_2134_8026 => 'Book Chapters (Aeronautical and Automotive Engineering)',
    col_2134_8176 => 'Pre-Prints (Design School)',
    col_2134_8258 => 'Closed Access PhD Theses (Aeronautical and Automotive Engineering)',
    col_2134_8261 => 'Closed Access PhD Theses (Chemical Engineering)',
    col_2134_8262 => 'Closed Access PhD Theses (Civil and Building Engineering)',
    col_2134_8263 => 'Closed Access PhD Theses (Electronic, Electrical and Systems Engineering)',
    col_2134_8264 => 'Closed Access PhD Theses (Materials)',
    col_2134_8265 => 'Closed Access PhD Theses (Mechanical and Manufacturing Engineering)',
    col_2134_8266 => 'Closed Access PhD Theses (Chemistry)',
    col_2134_8267 => 'Closed Access PhD Theses (Computer Science)',
    col_2134_8268 => 'Closed Access PhD Theses (Information Science)',
    col_2134_8269 => 'Closed Access PhD Theses (Maths)',
    col_2134_8270 => 'Closed Access PhD Theses (Physics)',
    col_2134_8273 => 'Closed Access PhD Theses (Business School)',
    col_2134_8274 => 'Closed Access PhD Theses (Economics)',
    col_2134_8275 => 'Closed Access PhD Theses (English and Drama)',
    col_2134_8280 => 'Closed Access PhD Theses (Geography)',
    col_2134_8281 => 'Closed Access PhD Theses (PHIR)',
    col_2134_8283 => 'Closed Access PhD Theses (Design School)',
    col_2134_8285 => 'Closed Access PhD theses (School of Arts)',
    col_2134_8352 => 'Research Reports (School of Sport, Exercise and Health Sciences)',
    col_2134_8354 => 'Research Reports (Design School)',
    col_2134_8513 => 'Teaching Award Case Studies (HEA Engineering Subject Centre)',
    col_2134_8535 => 'Published Articles (Mathematics Education Centre)',
    col_2134_8537 => 'Teaching Guides (HEA Engineering Subject Centre)',
    col_2134_8549 => 'Official Reports (PHIR)',
    col_2134_8582 => 'Conference Papers (Mathematics Education Centre)',
    col_2134_8628 => 'Closed Access (Mathematics Education Centre)',
    col_2134_8726 => 'Book Chapters (Mathematics Education Centre)',
    col_2134_8792 => 'Briefing Notes (WEDC)',
    col_2134_8804 => 'Published Articles (Research Office)',
    col_2134_8838 => 'Books (Mathematics Education Centre)',
    col_2134_8849 => 'Research Reports (Geography)',
    col_2134_8899 => 'LUPIN Imports',
    col_2134_9261 => 'Published Articles (HEA Engineering Subject Centre)',
    col_2134_9272 => 'MPhil Theses (Arts)',
    col_2134_9273 => 'Books (engCETL)',
    col_2134_9374 => 'Books (Centre for Engineering and Design Education)',
    col_2134_9375 => 'Conference Papers and Presentations (Centre for Engineering and Design Education)',
    col_2134_9376 => 'PhD Theses (Centre for Engineering and Design Education)',
    col_2134_9377 => 'Published Articles (Centre for Engineering and Design Education)',
    col_2134_9378 => 'Research Reports (Centre for Engineering and Design Education)',
    col_2134_9382 => 'Books (HEA Engineering Subject Centre)',
    col_2134_9411 => 'Book Chapters (WEDC)',
    col_2134_9415 => 'Conference Papers (HEA Engineering Subject Centre)',
    col_2134_9504 => 'Books (Civil and Building Engineering)',
    col_2134_9505 => 'Books (WEDC)',
    col_2134_9554 => 'Research Reports (Civil and Building Engineering)',
    col_2134_9555 => 'Research Reports (WEDC)',
    col_2134_9573 => 'Official Reports (Geography)',
    col_2134_9640 => 'Official Reports (Electronic, Electrical and Systems Engineering)',
    col_2134_9683 => 'Books (Social Sciences)',
    col_2134_9728 => 'Closed Access (CREST)',
    col_2134_9783 => 'Conference Papers (CICE)',
    col_2134_9971 => 'Official Reports (WEDC)',
    col_2134_10009 => 'Other (WEDC)',
    col_2134_10063 => 'Research Reports (Mathematics Education Centre)',
    col_2134_10064 => 'Books (Computer Science)',
    col_2134_10065 => 'Conference Papers (Maths)',
    col_2134_10122 => 'Working Papers (Computer Science)',
    col_2134_10242 => 'Closed Access MPhil Theses (Economics)',
    col_2134_10321 => 'Closed Access PhD theses (Sport, Exercise and Health Sciences)',
    col_2134_10334 => 'MSc Dissertations (Electronic, Electrical and Systems Engineering)',
    col_2134_10344 => 'MSc Dissertations (Aeronautical and Automotive Engineering)',
    col_2134_10365 => 'MSc Dissertations (Maths)',
    col_2134_10380 => 'MPhil Theses (Mechanical and Manufacturing Engineering)',
    col_2134_10385 => 'MPhil Theses (Materials)',
    col_2134_10391 => 'MPhil Theses (Maths)',
    col_2134_10408 => 'Closed Access MPhil Theses (English and Drama)',
    col_2134_10409 => 'MSc Dissertations (Computer Science)',
    col_2134_10416 => 'MPhil Theses (Chemical Engineering)',
    col_2134_10422 => 'Closed Access PhD theses (Social Sciences)',
    col_2134_10435 => 'Closed Access DSc Theses (Chemistry)',
    col_2134_10444 => 'MPhil theses (Physics)',
    col_2134_10462 => 'MPhil Theses (Chemistry)',
    col_2134_10496 => 'MPhil Theses (Economics)',
    col_2134_10501 => 'Closed Access MPhil Theses (Geography)',
    col_2134_10539 => 'MPhil Theses (Aeronautical and Automotive Engineering)',
    col_2134_10709 => 'MSc and MA Dissertations (Economics)',
    col_2134_10714 => 'Exhibitions (Arts)',
    col_2134_10726 => 'Graphs (Mathematics Education Centre)',
    col_2134_10814 => 'MPhil Theses (Civil and Building Engineering)',
    col_2134_10878 => 'MPhil Theses (Information Science)',
    col_2134_10974 => 'Closed Access MSc and MA Dissertations (Information Science )',
    col_2134_10994 => 'MPhil Theses (Electronic, Electrical and Systems Engineering)',
    col_2134_11093 => 'MPhil Theses (Geography)',
    col_2134_11097 => 'MPhil Theses (Computer Science)',
    col_2134_11222 => 'MPhil Theses (Social Sciences)',
    col_2134_11274 => 'Closed Access DLitt Theses (Geography)',
    col_2134_11497 => 'Book Chapters (Information Science)',
    col_2134_11562 => 'Official Reports (Mechanical and Manufacturing Engineering)',
    col_2134_11598 => 'Closed Access MPhil Theses (Electronic, Electrical and Systems Engineering)',
    col_2134_11726 => 'Closed Access DSc Theses (Civil and Building Engineering)',
    col_2134_12056 => 'MSc Dissertations (Physics)',
    col_2134_12062 => 'Closed Access DSc Theses (Mechanical and Manufacturing Engineering)',
    col_2134_12207 => 'Data Sets and Software (Civil and Building Engineering)',
    col_2134_12273 => 'Patents (Design School)',
    col_2134_12787 => 'Book Chapters (Physics)',
    col_2134_12968 => 'Official Reports (Information Science)',
    col_2134_13599 => 'Design Practice Research Case Studies (Design School)',
    col_2134_13677 => 'Other (English and Drama)',
    col_2134_13694 => 'Official Reports (Centre for Engineering and Design Education)',
    col_2134_13718 => 'Patents (Materials)',
    col_2134_13811 => 'Research Reports (IT Services)',
    col_2134_13837 => 'Pre-Prints (Aeronautical and Automotive Engineering)',
    col_2134_13889 => 'Data Sets and Software (Aeronautical and Automotive Engineering)',
    col_2134_13890 => 'Closed Access DSc Theses (Information Science)',
    col_2134_14000 => 'PhD Theses (CIM)',
    col_2134_14168 => 'Book Chapters (Maths)',
    col_2134_14230 => 'Research Films (Aeronautical and Automotive Engineering)',
    col_2134_14322 => 'MSc Dissertations (Geography)',
    col_2134_14394 => 'MPhil Theses (PHIR)',
    col_2134_14437 => 'Published Articles (CIM)',
    col_2134_14526 => 'Closed Access DSc Theses (Chemical Engineering)',
    col_2134_14534 => 'Closed Access MPhil Theses (PHIR)',
    col_2134_14537 => 'Other (PHIR)',
    col_2134_15121 => 'Closed Access MPhil Theses (Social Sciences)',
    col_2134_15174 => 'MPhil Theses (English and Drama)',
    col_2134_15191 => 'Published Theses (Chemistry)',
    col_2134_15527 => 'Published Articles (IT Services)',
    col_2134_15642 => 'Other (Social Sciences)',
    col_2134_15832 => 'Other (Arts)',
    col_2134_16088 => 'Books (Geography)',
    col_2134_16212 => 'Official Reports (Research Office)',
    col_2134_16239 => 'Other (Geography)',
    col_2134_16461 => 'Published Theses (Aeronautical and Automotive Engineering)',
    col_2134_16748 => 'Conference Papers (CIM)',
    col_2134_16767 => 'Closed Access (CIM)',
    col_2134_16769 => 'Official Reports (Economics)',
    col_2134_17062 => 'Other (Computer Science)',
    col_2134_17290 => 'Other (Business School)',
    col_2134_17646 => 'Official Reports (English and Drama)',
    col_2134_17647 => 'Official Reports (CIM)',
    col_2134_18138 => 'Working Papers (CRSP)',
    col_2134_18189 => 'Closed Access (Centre for Child and Family Research)',
    col_2134_18356 => 'MSc Dissertations (Social Sciences)',
    col_2134_18360 => 'Closed Access DSc Theses (Computer Science)',
    col_2134_18379 => 'Published Articles (Centre for Child and Family Research)',
    col_2134_18542 => 'Official Reports (Mathematics Education Centre)',
    col_2134_18826 => 'PhD Theses (WEDC)',
    col_2134_19017 => 'Published Articles (Loughborough University London)',
    col_2134_19301 => 'Technical Reports (Computer Science)',
    col_2134_19304 => 'Conference Papers (Loughborough University London)',
    col_2134_19416 => 'Closed Access (Loughborough University London)',
    col_2134_19422 => 'Published Theses (CREST)',
    col_2134_19493 => 'Published Articles (Sports Technology Research Group)',
    col_2134_19646 => 'DSc Theses (Mechanical and Manufacturing Engineering)',
    col_2134_19879 => 'Books (Arts)',
    col_2134_20093 => 'Closed Access PhD Theses (CIM)',
    col_2134_20207 => 'Patents (Electronic, Electrical and Systems Engineering)',
    col_2134_20213 => 'Book Chapters (Loughborough University London)',
};

1;
